@@ -1,6 +1,6 @@
-#!/usr/bin/env python
+#!/usr/bin/python
 
-## Copyright (C) 2007, 2008, 2009, 2010 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013 Red Hat, Inc.
 ## Copyright (C) 2008 Novell, Inc.
 ## Author: Tim Waugh <twaugh@redhat.com>
 
@@ -16,24 +16,25 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import threading
 import config
 import cups
-import gobject
-import gtk
+from gi.repository import GObject
+from gi.repository import GLib
+from gi.repository import Gdk
+from gi.repository import Gtk
 import Queue
 
+cups.require ("1.9.60")
+
 import authconn
 from debug import *
 import debug
+import gettext
+gettext.install(domain=config.PACKAGE, localedir=config.localedir, unicode=True)
 
-_ = lambda x: x
-N_ = lambda x: x
-def set_gettext_function (fn):
-    global _
-    _ = fn
 
 ######
 ###### An asynchronous libcups API using IPP with a separate worker
@@ -45,20 +46,21 @@ def set_gettext_function (fn):
 ###
 class _IPPConnectionThread(threading.Thread):
     def __init__ (self, queue, conn, reply_handler=None, error_handler=None,
-                  auth_handler=None, host=None, port=None, encryption=None):
+                  auth_handler=None, user=None, host=None, port=None,
+                  encryption=None):
                   
         threading.Thread.__init__ (self)
         self.setDaemon (True)
         self._queue = queue
         self._conn = conn
         self.host = host
-        self._port = port
+        self.port = port
         self._encryption = encryption
         self._reply_handler = reply_handler
         self._error_handler = error_handler
         self._auth_handler = auth_handler
         self._auth_queue = Queue.Queue (1)
-        self.user = None
+        self.user = user
         self._destroyed = False
         debugprint ("+%s" % self)
 
@@ -71,32 +73,31 @@ class _IPPConnectionThread(threading.Thread):
     def run (self):
         if self.host == None:
             self.host = cups.getServer ()
-        if self._port == None:
-            self._port = cups.getPort ()
+        if self.port == None:
+            self.port = cups.getPort ()
         if self._encryption == None:
             self._encryption = cups.getEncryption ()
 
-        self.user = cups.getUser ()
+        if self.user:
+            cups.setUser (self.user)
+        else:
+            self.user = cups.getUser ()
 
-        try:
-            cups.setPasswordCB2 (self._auth)
-        except AttributeError:
-            # Requires pycups >= 1.9.47.  Fall back to rubbish API.
-            cups.setPasswordCB (self._auth)
+        cups.setPasswordCB2 (self._auth)
 
         try:
             conn = cups.Connection (host=self.host,
-                                    port=self._port,
+                                    port=self.port,
                                     encryption=self._encryption)
-        except RuntimeError, e:
+            self._reply (None)
+        except RuntimeError as e:
+            conn = None
             self._error (e)
-            return
-
-        self._reply (None)
 
         while True:
             # Wait to find out what operation to try.
             debugprint ("Awaiting further instructions")
+            self.idle = self._queue.empty ()
             item = self._queue.get ()
             debugprint ("Next task: %s" % repr (item))
             if item == None:
@@ -104,6 +105,7 @@ class _IPPConnectionThread(threading.Thread):
                 self._queue.task_done ()
                 break
 
+            self.idle = False
             (fn, args, kwds, rh, eh, ah) = item
             if rh != False:
                 self._reply_handler = rh
@@ -117,25 +119,20 @@ class _IPPConnectionThread(threading.Thread):
                 self.user = args[0]
                 cups.setUser (self.user)
                 debugprint ("Set user=%s; reconnecting..." % self.user)
-                try:
-                    cups.setPasswordCB2 (self._auth)
-                except AttributeError:
-                    # Requires pycups >= 1.9.47.  Fall back to rubbish API.
-                    cups.setPasswordCB (self._auth)
+                cups.setPasswordCB2 (self._auth)
 
                 try:
                     conn = cups.Connection (host=self.host,
-                                            port=self._port,
+                                            port=self.port,
                                             encryption=self._encryption)
                     debugprint ("...reconnected")
-                except RuntimeError, e:
+
+                    self._queue.task_done ()
+                    self._reply (None)
+                except RuntimeError as e:
                     debugprint ("...failed")
                     self._queue.task_done ()
                     self._error (e)
-                    break
-
-                self._queue.task_done ()
-                self._reply (None)
 
                 continue
 
@@ -151,8 +148,8 @@ class _IPPConnectionThread(threading.Thread):
 
                 debugprint ("...success")
                 self._reply (result)
-            except Exception, e:
-                debugprint ("...failure")
+            except Exception as e:
+                debugprint ("...failure (%s)" % repr (e))
                 self._error (e)
 
             self._queue.task_done ()
@@ -167,46 +164,48 @@ class _IPPConnectionThread(threading.Thread):
         del self._auth_queue
         del conn
 
-        try:
-            cups.setPasswordCB2 (None)
-        except AttributeError:
-            # Requires pycups >= 1.9.47.  Fall back to rubbish API.
-            cups.setPasswordCB (lambda x: '')
+        cups.setPasswordCB2 (None)
 
     def _auth (self, prompt, conn=None, method=None, resource=None):
         def prompt_auth (prompt):
+            Gdk.threads_enter ()
             if conn == None:
                 self._auth_handler (prompt, self._conn)
             else:
                 self._auth_handler (prompt, self._conn, method, resource)
 
+            Gdk.threads_leave ()
             return False
 
         if self._auth_handler == None:
             return ""
 
-        gobject.idle_add (prompt_auth, prompt)
+        GLib.idle_add (prompt_auth, prompt)
         password = self._auth_queue.get ()
         return password
 
     def _reply (self, result):
         def send_reply (handler, result):
             if not self._destroyed:
+                Gdk.threads_enter ()
                 handler (self._conn, result)
+                Gdk.threads_leave ()
             return False
 
         if not self._destroyed and self._reply_handler:
-            gobject.idle_add (send_reply, self._reply_handler, result)
+            GLib.idle_add (send_reply, self._reply_handler, result)
 
     def _error (self, exc):
         def send_error (handler, exc):
             if not self._destroyed:
+                Gdk.threads_enter ()
                 handler (self._conn, exc)
+                Gdk.threads_leave ()
             return False
 
         if not self._destroyed and self._error_handler:
             debugprint ("Add %s to idle" % self._error_handler)
-            gobject.idle_add (send_error, self._error_handler, exc)
+            GLib.idle_add (send_error, self._error_handler, exc)
 
 ###
 ### This is the user-visible class.  Although it does not inherit from
@@ -228,8 +227,8 @@ class IPPConnection:
     """
 
     def __init__ (self, reply_handler=None, error_handler=None,
-                  auth_handler=None, host=None, port=None, encryption=None,
-                  parent=None):
+                  auth_handler=None, user=None, host=None, port=None,
+                  encryption=None, parent=None):
         debugprint ("New IPPConnection")
         self._parent = parent
         self.queue = Queue.Queue ()
@@ -237,7 +236,7 @@ class IPPConnection:
                                             reply_handler=reply_handler,
                                             error_handler=error_handler,
                                             auth_handler=auth_handler,
-                                            host=host, port=port,
+                                            user=user, host=host, port=port,
                                             encryption=encryption)
         self.thread.start ()
 
@@ -260,19 +259,28 @@ class IPPConnection:
 
     def destroy (self):
         debugprint ("DESTROY: %s" % self)
+        for binding in self.bindings:
+            delattr (self, binding)
+
         if self.thread.isAlive ():
+            GLib.timeout_add_seconds (1, self._reap_thread)
+
+    def _reap_thread (self):
+        if self.thread.idle:
             debugprint ("Putting None on the task queue")
             self.queue.put (None)
             self.queue.join ()
+            return False
 
-        for binding in self.bindings:
-            delattr (self, binding)
+        debugprint ("Thread %s still processing tasks" % self.thread)
+        return True
 
     def set_auth_info (self, password):
         """Call this from your auth_handler function."""
         self.thread.set_auth_info (password)
 
     def reconnect (self, user, reply_handler=None, error_handler=None):
+        debugprint ("Reconnect...")
         self.queue.put ((True, (user,), {},
                          reply_handler, error_handler, False))
 
@@ -319,6 +327,7 @@ class _IPPAuthOperation:
         self._client_kwds = kwds
         self._client_reply_handler = reply_handler
         self._client_error_handler = error_handler
+
         debugprint ("+%s" % self)
 
     def __del__ (self):
@@ -340,11 +349,14 @@ class _IPPAuthOperation:
             return self._reconnect_error (conn, exc)
 
         if self._cancel:
+            debugprint ("%s (_error_handler): canceled so chaining up" % self)
             return self._error (exc)
 
         if self._reconnect:
             self._reconnect = False
             self._reconnected = True
+            debugprint ("%s (_error_handler): reconnecting (as %s)..." %
+                        (self, self._user))
             conn.reconnect (self._user,
                             reply_handler=self._reconnect_reply,
                             error_handler=self._reconnect_error)
@@ -354,7 +366,8 @@ class _IPPAuthOperation:
         if type (exc) == cups.IPPError:
             (e, m) = exc.args
             if (e == cups.IPP_NOT_AUTHORIZED or
-                e == cups.IPP_FORBIDDEN):
+                e == cups.IPP_FORBIDDEN or
+                e == cups.IPP_AUTHENTICATION_CANCELED):
                 forbidden = (e == cups.IPP_FORBIDDEN)
             elif e == cups.IPP_SERVICE_UNAVAILABLE:
                 return self._reconnect_error (conn, exc)
@@ -371,6 +384,10 @@ class _IPPAuthOperation:
             return self._error (exc)
 
         # Not authorized.
+        if forbidden:
+            debugprint ("%s (_error_handler): forbidden" % self)
+        else:
+            debugprint ("%s (_error_handler): not authorized" % self)
 
         if (self._try_as_root and
             self._user != 'root' and
@@ -385,7 +402,6 @@ class _IPPAuthOperation:
             # first.
             debugprint ("Authentication: Try as root")
             self._user = "root"
-            self._try_as_root = False
             conn.reconnect (self._user,
                             reply_handler=self._reconnect_reply,
                             error_handler=self._reconnect_error)
@@ -397,11 +413,38 @@ class _IPPAuthOperation:
             return self._error (exc)
 
         # Now reconnect and retry.
+        host = conn.thread.host
+        port = conn.thread.port
+        authconn.global_authinfocache.remove_auth_info (host=host,
+                                                        port=port)
+        self._use_password = ''
+        debugprint ("%s (_error_handler): reconnecting (as %s)..." %
+                    (self, self._user))
         conn.reconnect (self._user,
                         reply_handler=self._reconnect_reply,
                         error_handler=self._reconnect_error)
 
     def auth_handler (self, prompt, conn, method=None, resource=None):
+        if self._auth_called == False:
+            if self._user == None:
+                self._user = cups.getUser()
+            if self._user:
+                host = conn.thread.host
+                port = conn.thread.port
+                creds = authconn.global_authinfocache.lookup_auth_info (host=host,
+                                                                        port=port)
+                if creds:
+                    if creds[0] == self._user:
+                        self._use_password = creds[1]
+                        self._reconnected = True
+                    del creds
+        else:
+            host = conn.thread.host
+            port = conn.thread.port
+            authconn.global_authinfocache.remove_auth_info (host=host,
+                                                            port=port)
+            self._use_password = ''
+
         self._auth_called = True
         if self._reconnected:
             debugprint ("Supplying password after reconnection")
@@ -416,11 +459,11 @@ class _IPPAuthOperation:
 
         # If we've previously prompted, explain why we're prompting again.
         if self._dialog_shown:
-            d = gtk.MessageDialog (self._conn.parent,
-                                   gtk.DIALOG_MODAL |
-                                   gtk.DIALOG_DESTROY_WITH_PARENT,
-                                   gtk.MESSAGE_ERROR,
-                                   gtk.BUTTONS_CLOSE,
+            d = Gtk.MessageDialog (self._conn.parent,
+                                   Gtk.DialogFlags.MODAL |
+                                   Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                                   Gtk.MessageType.ERROR,
+                                   Gtk.ButtonsType.CLOSE,
                                    _("Not authorized"))
             d.format_secondary_text (_("The password may be incorrect."))
             d.run ()
@@ -437,9 +480,11 @@ class _IPPAuthOperation:
             d = authconn.AuthDialog (title=title,
                                      parent=conn.parent)
 
-        d.set_prompt (prompt)
-        d.set_auth_info ([self._user, ''])
-        d.field_grab_focus ('password')
+        d.set_prompt ('')
+        if self._user == None:
+            self._user = cups.getUser()
+        d.set_auth_info (['', ''])
+        d.field_grab_focus ('username')
         d.set_keep_above (True)
         d.show_all ()
         d.connect ("response", self._on_auth_dialog_response)
@@ -457,13 +502,21 @@ class _IPPAuthOperation:
 
     def _on_auth_dialog_response (self, dialog, response):
         (user, password) = dialog.get_auth_info ()
+        if user == '':
+            user = self._user;
+        authconn.global_authinfocache.cache_auth_info ((user,
+                                                        password),
+                                                       host=self._conn.thread.host,
+                                                       port=self._conn.thread.port)
         self._dialog = dialog
         dialog.hide ()
 
-        if (response == gtk.RESPONSE_CANCEL or
-            response == gtk.RESPONSE_DELETE_EVENT):
+        if (response == Gtk.ResponseType.CANCEL or
+            response == Gtk.ResponseType.DELETE_EVENT):
             self._cancel = True
             self._conn.set_auth_info ('')
+            authconn.global_authinfocache.remove_auth_info (host=self._conn.thread.host,
+                                                            port=self._conn.thread.port)
             debugprint ("Auth canceled")
             return
 
@@ -502,11 +555,11 @@ class _IPPAuthOperation:
         else:
             msg = _("CUPS server error (%s)") % op
 
-        d = gtk.MessageDialog (self._conn.parent,
-                               gtk.DIALOG_MODAL |
-                               gtk.DIALOG_DESTROY_WITH_PARENT,
-                               gtk.MESSAGE_ERROR,
-                               gtk.BUTTONS_NONE,
+        d = Gtk.MessageDialog (self._conn.parent,
+                               Gtk.DialogFlags.MODAL |
+                               Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                               Gtk.MessageType.ERROR,
+                               Gtk.ButtonsType.NONE,
                                msg)
 
         if self._client_fn == None and type (exc) == RuntimeError:
@@ -519,24 +572,35 @@ class _IPPAuthOperation:
 
         d.format_secondary_text (_("There was an error during the "
                                    "CUPS operation: '%s'." % message))
-        d.add_buttons (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                       _("Retry"), gtk.RESPONSE_OK)
-        d.set_default_response (gtk.RESPONSE_OK)
+        d.add_buttons (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                       _("Retry"), Gtk.ResponseType.OK)
+        d.set_default_response (Gtk.ResponseType.OK)
         d.connect ("response", self._on_retry_server_error_response)
+        debugprint ("%s (_reconnect_error): presenting error dialog (%s; %s)" %
+                    (self, msg, message))
+        d.show ()
 
     def _on_retry_server_error_response (self, dialog, response):
         dialog.destroy ()
-        if response == gtk.RESPONSE_OK:
-            self.reconnect (self._conn.thread.user,
-                            reply_handler=self._reconnect_reply,
-                            error_handler=self._reconnect_error)
+        if response == Gtk.ResponseType.OK:
+            debugprint ("%s: got retry response, reconnecting (as %s)..." %
+                        (self, self._conn.thread.user))
+            self._conn.reconnect (self._conn.thread.user,
+                                  reply_handler=self._reconnect_reply,
+                                  error_handler=self._reconnect_error)
         else:
+            debugprint ("%s: got cancel response" % self)
             self._error (cups.IPPError (0, _("Operation canceled")))
 
     def _error (self, exc):
+        debugprint ("%s (_error): handling %s" % (self, repr (exc)))
         if self._client_error_handler:
+            debugprint ("%s (_error): calling %s" %
+                        (self, self._client_error_handler))
             self._client_error_handler (self._conn, exc)
             self._destroy ()
+        else:
+            debugprint ("%s (_error): no client error handler set" % self)
 
 ###
 ### The user-visible class.
@@ -551,15 +615,23 @@ class IPPAuthConnection(IPPConnection):
         self.try_as_root = try_as_root
         self.semantic = semantic
 
+        user = None
+        creds = authconn.global_authinfocache.lookup_auth_info (host=host,
+                                                                port=port)
+        if creds:
+            if creds[0] != 'root' or try_as_root:
+                user = creds[0]
+            del creds
+
         # The "connect" operation.
         op = _IPPAuthOperation (reply_handler, error_handler, self)
         IPPConnection.__init__ (self, reply_handler=reply_handler,
                                 error_handler=op.error_handler,
-                                auth_handler=op.auth_handler, host=host,
-                                port=port, encryption=encryption)
+                                auth_handler=op.auth_handler, user=user,
+                                host=host, port=port, encryption=encryption)
 
     def destroy (self):
-        del self.semantic
+        self.semantic = None
         IPPConnection.destroy (self)
 
     def _call_function (self, fn, *args, **kwds):
@@ -584,22 +656,21 @@ class IPPAuthConnection(IPPConnection):
 
 if __name__ == "__main__":
     # Demo
-    import gtk
     set_debugging (True)
-    gobject.threads_init ()
+    GObject.threads_init ()
     class UI:
         def __init__ (self):
-            w = gtk.Window ()
+            w = Gtk.Window ()
             w.connect ("destroy", self.destroy)
-            b = gtk.Button ("Connect")
+            b = Gtk.Button ("Connect")
             b.connect ("clicked", self.connect_clicked)
-            vbox = gtk.VBox ()
-            vbox.pack_start (b)
+            vbox = Gtk.VBox ()
+            vbox.pack_start (b, False, False, 0)
             w.add (vbox)
-            self.get_devices_button = gtk.Button ("Get Devices")
+            self.get_devices_button = Gtk.Button ("Get Devices")
             self.get_devices_button.connect ("clicked", self.get_devices)
             self.get_devices_button.set_sensitive (False)
-            vbox.pack_start (self.get_devices_button)
+            vbox.pack_start (self.get_devices_button, False, False, 0)
             self.conn = None
             w.show_all ()
 
@@ -609,7 +680,7 @@ if __name__ == "__main__":
             except AttributeError:
                 pass
 
-            gtk.main_quit ()
+            Gtk.main_quit ()
 
         def connect_clicked (self, button):
             if self.conn:
@@ -619,11 +690,11 @@ if __name__ == "__main__":
                                            error_handler=self.connect_failed)
 
         def connected (self, conn, result):
-            debugprint ("Success: %s" % result)
+            debugprint ("Success: %s" % repr (result))
             self.get_devices_button.set_sensitive (True)
 
         def connect_failed (self, conn, exc):
-            debugprint ("Exc %s" % exc)
+            debugprint ("Exc %s" % repr (exc))
             self.get_devices_button.set_sensitive (False)
             self.conn.destroy ()
 
@@ -638,7 +709,7 @@ if __name__ == "__main__":
                 debugprint ("Ignoring stale reply")
                 return
 
-            debugprint ("Got devices: %s" % result)
+            debugprint ("Got devices: %s" % repr (result))
             self.get_devices_button.set_sensitive (True)
 
         def get_devices_error (self, conn, exc):
@@ -646,8 +717,8 @@ if __name__ == "__main__":
                 debugprint ("Ignoring stale error")
                 return
 
-            debugprint ("Error getting devices: %s" % exc)
+            debugprint ("Error getting devices: %s" % repr (exc))
             self.get_devices_button.set_sensitive (True)
 
     UI ()
-    gtk.main ()
+    Gtk.main ()
