@@ -247,6 +247,8 @@ class Connection:
                 break
             except cups.IPPError as e:
                 (e, m) = e.args
+                if isinstance(m, bytes):
+                    m = m.decode('utf-8', 'replace')
                 if self._use_pk and m == 'pkcancel':
                     raise cups.IPPError (0, _("Operation canceled"))
 
@@ -302,7 +304,7 @@ class Connection:
                                msg)
                                
         d.format_secondary_text (_("There was an error during the "
-                                   "CUPS operation: '%s'." % message))
+                                   "CUPS operation: '%s'.") % message)
         d.add_buttons (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                        _("Retry"), Gtk.ResponseType.OK)
         d.set_default_response (Gtk.ResponseType.OK)
