@@ -54,6 +54,8 @@ elif len (sys.argv) == 2:
             attrs = c.getPrinterAttributes (sys.argv[1])
         except cups.IPPError as e:
             (e, m) = e.args
+            if isinstance(m, bytes):
+                m = m.decode('utf-8', 'replace')
             print "Error getting printer attibutes: %s" % m
             sys.exit (1)
 
@@ -80,7 +82,7 @@ if devices == None:
             print "Run this as root to examine IDs from attached devices."
             sys.exit (1)
 
-        if e == cups.IPP_NOT_AUTHORIZED:
+        if e in (cups.IPP_NOT_AUTHORIZED, cups.IPP_AUTHENTICATION_CANCELED):
             print "Not authorized."
             sys.exit (1)
 
