@@ -54,7 +54,7 @@ def show_IPP_Error(exception, message, parent=None):
     else:
         # cups.IPPError message is utf-8 encoded
         if isinstance(message, bytes):
-            message = message.decode ('utf-8')
+            message = message.decode ('utf-8', 'replace')
         title = _("CUPS server error")
         text = _("There was an error during the CUPS "
                  "operation: '%s'.") % message
