@@ -1,6 +1,6 @@
 #!/usr/bin/python
 
-## Copyright (C) 2007, 2008, 2009, 2010 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2012, 2013 Red Hat, Inc.
 ## Authors:
 ##  Tim Waugh <twaugh@redhat.com>
 ##  Jiri Popelka <jpopelka@redhat.com>
@@ -17,11 +17,13 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import cups
 import os
-from gettext import gettext as _
+import config
+import gettext
+gettext.install(domain=config.PACKAGE, localedir=config.localedir, unicode=True)
 
 class StateReason:
     REPORT=1
@@ -29,9 +31,9 @@ class StateReason:
     ERROR=3
 
     LEVEL_ICON={
-        REPORT: "dialog-info",
-        WARNING: "dialog-warning",
-        ERROR: "dialog-error"
+        REPORT: "gtk-dialog-info",
+        WARNING: "gtk-dialog-warning",
+        ERROR: "gtk-dialog-error"
         }
 
     def __init__(self, printer, reason, ppdcache=None):
@@ -148,7 +150,7 @@ class StateReason:
                 except RuntimeError:
                     pass
 
-            text = _("Printer '%s': '%s'.") % (self.get_printer (), reason)
+            text = (_("Printer '%s': '%s'.") % (self.get_printer (), reason))
         return (title, text)
 
     def get_tuple (self):
