@@ -18,7 +18,9 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+
+from gi.repository import Gtk
 
 import cups
 import os
@@ -36,7 +38,7 @@ except:
 class CheckNetworkServerSanity(Question):
     def __init__ (self, troubleshooter):
         Question.__init__ (self, troubleshooter, "Check network server sanity")
-        troubleshooter.new_page (gtk.Label (), self)
+        troubleshooter.new_page (Gtk.Label (), self)
 
     def display (self):
         # Collect useful information.
@@ -156,7 +158,8 @@ class CheckNetworkServerSanity(Question):
             except NameError:
                 # No smbc support
                 pass
-            except RuntimeError, (e, s):
+            except RuntimeError as e:
+                (e, s) = e.args
                 self.answers['remote_server_smb_shares'] = (e, s)
 
             if context != None and answers.has_key ('cups_printer_dict'):
@@ -173,7 +176,8 @@ class CheckNetworkServerSanity(Question):
                                               parent=parent)
                     f  = self.op.run ()
                     accessible = True
-                except RuntimeError, (e, s):
+                except RuntimeError as e:
+                    (e, s) = e.args
                     accessible = (e, s)
 
                 self.answers['remote_server_smb_share_anon_access'] = accessible
