@@ -18,7 +18,7 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import glob
 import os
@@ -26,10 +26,12 @@ import subprocess
 from timedops import TimedSubprocess
 import urllib
 from base import *
+from gi.repository import Gtk
+
 class CheckUSBPermissions(Question):
     def __init__ (self, troubleshooter):
         Question.__init__ (self, troubleshooter, "Check USB permissions")
-        troubleshooter.new_page (gtk.Label (), self)
+        troubleshooter.new_page (Gtk.Label (), self)
 
     def display (self):
         self.answers = {}
@@ -57,13 +59,16 @@ class CheckUSBPermissions(Question):
         if not os.access (GETFACL, os.X_OK):
             return False
 
+        new_environ = os.environ.copy()
+        new_environ['LC_ALL'] = "C"
+
         # Run lsusb
         parent = self.troubleshooter.get_window ()
         try:
             self.op = TimedSubprocess (parent=parent,
-                                       args="LC_ALL=C " + LSUSB + " -v",
+                                       args=[LSUSB, "-v"],
                                        close_fds=True,
-                                       shell=True,
+                                       env=new_environ,
                                        stdin=file("/dev/null"),
                                        stdout=subprocess.PIPE,
                                        stderr=subprocess.PIPE)
@@ -141,10 +146,9 @@ class CheckUSBPermissions(Question):
         for path in paths:
             try:
                 self.op = TimedSubprocess (parent=parent,
-                                           args="LC_ALL=C %s %s" % (GETFACL,
-                                                                    path),
+                                           args=[GETFACL, path],
                                            close_fds=True,
-                                           shell=True,
+                                           env=new_environ,
                                            stdin=file("/dev/null"),
                                            stdout=subprocess.PIPE,
                                            stderr=subprocess.PIPE)
